@@ -77,7 +77,7 @@ static void cd_recorder_init       (CDRecorder      *cdrecorder);
 static int cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 		CDDrive *recorder, GList *tracks,
 		int speed, gboolean eject, gboolean blank,
-		gboolean dummy_write);
+		gboolean dummy_write, gboolean disc_at_once);
 static int cd_recorder_write_growisofs (CDRecorder *cdrecorder,
 		CDDrive *recorder, GList *tracks,
 		int speed, gboolean eject, gboolean blank,
@@ -415,7 +415,8 @@ cd_recorder_write_tracks (CDRecorder *cdrecorder,
 		       int speed,
 		       gboolean eject,
 		       gboolean blank,
-		       gboolean dummy_write)
+		       gboolean dummy_write,
+		       gboolean disc_at_once)
 		       /* TODO: GError **error */
 {
 	g_return_val_if_fail (tracks != NULL, RESULT_ERROR);
@@ -431,7 +432,8 @@ cd_recorder_write_tracks (CDRecorder *cdrecorder,
 	} else {
 		return cd_recorder_write_cdrecord (cdrecorder,
 						   recorder, tracks, speed,
-						   eject, blank, dummy_write);
+						   eject, blank, dummy_write,
+						   disc_at_once);
 	}
 }
 
@@ -595,7 +597,8 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 		       int speed,
 		       gboolean eject,
 		       gboolean blank,
-		       gboolean dummy_write)
+		       gboolean dummy_write,
+		       gboolean disc_at_once)
 {
 	GPtrArray *argv;
 	char *speed_str, *dev_str;
@@ -625,6 +628,9 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 	if (blank) {
 		g_ptr_array_add (argv, "blank=fast");
 	}
+	if (disc_at_once) {
+		g_ptr_array_add (argv, "-dao");
+	}
 	g_ptr_array_add (argv, "-v");
 
 	l = tracks;
@@ -637,6 +643,7 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 			g_ptr_array_add (argv, track->data.filename);
 			break;
 		case TRACK_TYPE_AUDIO:
+			g_ptr_array_add (argv, "-copy");
 			g_ptr_array_add (argv, "-audio");
 			g_ptr_array_add (argv, "-pad");
 			g_ptr_array_add (argv, track->audio.filename);
