@@ -46,11 +46,14 @@
 #include <gtk/gtkmessagedialog.h>
 
 #include "nautilus-cd-burner.h"
+#include "make-iso.h"
 
 #ifndef HAVE_MKDTEMP
 #include "mkdtemp.h"
 #endif
 
+extern gboolean is_audio (char *mime_type);
+
 static void
 write_all (int fd, char *buf, int len)
 {
@@ -215,6 +218,23 @@ get_backing_file (const char *virtual_path)
 	return NULL;
 }
 
+static gboolean
+ask_disable_joliet (GtkWindow *parent)
+{
+	GtkWidget *dialog;
+	int res;
+
+	dialog = gtk_message_dialog_new (parent,
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_OK_CANCEL,
+					 _("Some files don't have a suitable name for a Windows-compatible CD.\nDo you want to continue with Windows compatibility disabled?"));
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Windows compatibility"));
+	res = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	return (res == GTK_RESPONSE_OK);
+}
+
 struct mkisofs_state {
 	FILE *graft_file;
 	char *emptydir;
@@ -466,8 +486,12 @@ cancel_mkisofs (gpointer data)
 	g_main_loop_quit (mkisofs_output->loop);
 }
 
+/**
+ * Create an ISO image in filename from the data files in burn:///
+ */
 int
-make_iso (const char *filename, const char *label)
+make_iso (const char *filename, const char *label,
+		gboolean warn_low_space, gboolean use_joliet)
 {
 	GnomeVFSURI *uri;
 	char *filelist = NULL;
@@ -481,7 +505,7 @@ make_iso (const char *filename, const char *label)
 	struct mkisofs_output mkisofs_output;
 	GIOChannel *channel;
 	guint stdout_tag, stderr_tag;
-	char *stdout_data;
+	char *stdout_data, *stderr_data;
 	char *dirname;
 	int exit_status, res;
 	unsigned long iso_size;
@@ -540,24 +564,28 @@ make_iso (const char *filename, const char *label)
 		mkisofs_output.result = RESULT_ERROR;
 		goto cleanup;
 	}
-	
+
+retry:
 	i = 0;
 	argv[i++] = "mkisofs";
 	argv[i++] = "-r";
-	argv[i++] = "-J";
+	if (use_joliet) {
+		argv[i++] = "-J";
+	}
+	argv[i++] = "-q";
 	argv[i++] = "-graft-points";
 	argv[i++] = "-path-list";
 	argv[i++] = filelist;
 	argv[i++] = "-print-size";
 	argv[i++] = NULL;
-	
+
 	if (!g_spawn_sync (NULL,
 			   (char **)argv,
 			   NULL,
-			   G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+			   G_SPAWN_SEARCH_PATH,
 			   NULL, NULL,
 			   &stdout_data,
-			   NULL,
+			   &stderr_data,
 			   &exit_status,
 			   NULL)) {
 		g_warning ("mkisofs command failed: %s\n", error->message);
@@ -566,6 +594,22 @@ make_iso (const char *filename, const char *label)
 		mkisofs_output.result = RESULT_ERROR;
 		goto cleanup;
 	}
+
+	if (exit_status != 0 && use_joliet) {
+		if (strstr (stderr_data, "Joliet tree sort failed.") != NULL) {
+			g_free (stdout_data);
+			g_free (stderr_data);
+			if (ask_disable_joliet (cd_progress_get_window ())) {
+				use_joliet = FALSE;
+				goto retry;
+			} else {
+				mkisofs_output.result = RESULT_ERROR;
+				goto cleanup;
+			}
+		}
+	}
+
+	g_free (stderr_data);
 	iso_size = atol (stdout_data); /* blocks of 2048 bytes */
 	g_free (stdout_data);
 
@@ -575,28 +619,38 @@ make_iso (const char *filename, const char *label)
 #else
 	res = statfs (dirname, &statfs_buf);
 #endif
-	g_free (dirname);
 	if (res == -1) {
 		g_warning ("Cannot get free space at %s\n", dirname);
+		g_free (dirname);
 	} else if (iso_size / statfs_buf.f_bsize >= statfs_buf.f_bavail / 2048) {
-		dialog = gtk_message_dialog_new (cd_progress_get_window (),
-						 GTK_DIALOG_DESTROY_WITH_PARENT,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("Not enough space to store CD image (%ld Megabytes needed)"),
-						 iso_size / 512);
-		gtk_window_set_title (GTK_WINDOW (dialog),
-				      _("Not enough space"));
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-		mkisofs_output.result = RESULT_ERROR;
+		g_free (dirname);
+		if (warn_low_space) {
+			dialog = gtk_message_dialog_new (cd_progress_get_window (),
+							 GTK_DIALOG_DESTROY_WITH_PARENT,
+							 GTK_MESSAGE_ERROR,
+							 GTK_BUTTONS_CLOSE,
+							 ngettext("Not enough space to store CD image (%ld Megabyte needed)",
+								  "Not enough space to store CD image (%ld Megabytes needed)",
+								  iso_size / 512),
+							 iso_size / 512);
+			gtk_window_set_title (GTK_WINDOW (dialog),
+					      _("Not enough space"));
+			gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy (dialog);
+			mkisofs_output.result = RESULT_ERROR;
+		} else {
+			mkisofs_output.result = RESULT_RETRY;
+		}
+
 		goto cleanup;
 	}
 
 	i = 0;
 	argv[i++] = "mkisofs";
 	argv[i++] = "-r";
-	argv[i++] = "-J";
+	if (use_joliet) {
+		argv[i++] = "-J";
+	}
 	argv[i++] = "-graft-points";
 	argv[i++] = "-path-list";
 	argv[i++] = filelist;
@@ -671,41 +725,43 @@ make_iso (const char *filename, const char *label)
 	return mkisofs_output.result;
 }
 
-gint 
-build_list_of_music_arguments (char *uri, int *argc, const char **argv, int max_num_args)
+/**
+ * Build a GList* of Track pointers from the audio tracks in burn:///
+ */
+GList *
+build_list_of_audio_tracks (void)
 {
+	GList *list;
 	GnomeVFSFileInfo *info;
 	GnomeVFSDirectoryHandle *handle;
 	GnomeVFSResult result;
 	char *backing;
-	gint count;
-
-	count = 0;
 	
-	result = gnome_vfs_directory_open (&handle, uri, GNOME_VFS_FILE_INFO_FOLLOW_LINKS | GNOME_VFS_FILE_INFO_GET_MIME_TYPE);
+	result = gnome_vfs_directory_open (&handle, "burn:///", GNOME_VFS_FILE_INFO_FOLLOW_LINKS | GNOME_VFS_FILE_INFO_GET_MIME_TYPE);
 	if (result != GNOME_VFS_OK) {
 		return 0;
 	}
 	
 	info = gnome_vfs_file_info_new ();
 
+	list = NULL;
 	while (TRUE) {
 		result = gnome_vfs_directory_read_next (handle, info);
 		if (result != GNOME_VFS_OK)
 			break;
 
 		if (is_audio (info->mime_type)) {
+			Track *track;
 			backing = get_backing_file (info->name);
-			argv[(*argc)++] = backing;
-			count++;
-		}
-		
-		if (count == max_num_args) {
-			break;		
+			track = g_new0 (Track, 1);
+			track->type = TRACK_TYPE_AUDIO;
+			track->audio.filename = backing;
+			track->audio.cdtext = NULL;
+			list = g_list_prepend (list, track);
 		}
 	}
 
 	gnome_vfs_directory_close (handle);
 	gnome_vfs_file_info_unref (info);
-	return count;
+	return g_list_reverse (list);
 }
