@@ -537,6 +537,15 @@ estimate_size (char *uri)
 				   (GnomeVFSDirectoryVisitFunc) estimate_size_callback,
 				   &size);
 
+	if (size == 0) {
+		GList *list;
+
+		if (gnome_vfs_directory_list_load (&list, uri, GNOME_VFS_FILE_INFO_FIELDS_NONE) == GNOME_VFS_OK) {
+			size = 1;
+			gnome_vfs_file_info_list_free (list);
+		}
+	}
+
 	return size;
 }
 
@@ -619,108 +628,81 @@ display_error_dialog (GtkWindow *win, const gchar *msg)
 }
 
 static void
-file_sel_ok_cb (GtkWidget *widget, gpointer data)
+file_chooser_response_cb (GtkWidget *chooser, gint response, gpointer data)
 {
-	GtkFileSelection *fsel;
-	const gchar *file_name;
-	gint name_len;
+	gchar *file_name;
+	char **file_name_out;
 
-	fsel = (GtkFileSelection *)data;
+	file_name_out = data;
 
-	file_name = gtk_file_selection_get_filename (fsel);
+	if (response != GTK_RESPONSE_OK) {
+		if (response != GTK_RESPONSE_DELETE_EVENT) {
+			gtk_widget_destroy (chooser);
+		}
+		gtk_main_quit ();
 
-	name_len = strlen (file_name);
-
-	if (name_len < 1)
 		return;
+	}
+
+	file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+
+	if (strlen (file_name) < 1) {
+		g_free (file_name);
+		return;
+	}
 
 	/* Change into directory if that's what user selected */
 	if (g_file_test (file_name, G_FILE_TEST_IS_DIR)) {
-		gchar *dir_name;
-
-		if (file_name [name_len - 1] != '/') {
-			/* The file selector needs a '/' at the end of a 
-			 * directory name 
-			 */
-			dir_name = g_strconcat (file_name, "/", NULL);
-		} else {
-			dir_name = g_strdup (file_name);
-		}
-		gtk_file_selection_set_filename (fsel, dir_name);
-		g_free (dir_name);
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
+						     file_name);
+		g_free (file_name);
 	} else {
 		if (g_file_test (file_name, G_FILE_TEST_EXISTS)) {
-			if (!overwrite_existing_file (GTK_WINDOW (fsel), file_name)) {
+			if (!overwrite_existing_file (GTK_WINDOW (chooser), file_name)) {
+				g_free (file_name);
 				return;
 			}
 		}
 
-		gtk_widget_hide (GTK_WIDGET (fsel));
-		g_object_set_data (G_OBJECT (fsel), 
-				   "file-name", 
-				   g_strdup (file_name));
+		gtk_widget_destroy (GTK_WIDGET (chooser));
+		
+		g_free (*file_name_out);
+		*file_name_out = file_name;
+		
 		gtk_main_quit ();
 	}
 }
 
-static void
-file_sel_cancel_cb (GtkWidget *widget, gpointer data)
-{
-	gtk_widget_hide (GTK_WIDGET (data));
-	gtk_main_quit ();
-
-	g_object_set_data (G_OBJECT (data),
-			   "file-name",
-			   NULL);
-}
-
-static gint
-file_sel_delete_event_cb (GtkWidget *widget, GdkEventAny *event, gpointer data)
-{
-	gtk_widget_hide (widget);
-	gtk_main_quit ();
-
-	g_object_set_data (G_OBJECT (data),
-			   "file-name",
-			   NULL);
-	return TRUE;
-}
-
 static char *
 select_iso_filename (void)
 {
-	GtkWidget *filesel;
-	gpointer data;
+	GtkWidget *chooser;
+	char *filename;
 
-	filesel = gtk_file_selection_new (_("Choose a filename for the cdrom image"));
-	gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
-					 "~/image.iso");
+	chooser = gtk_file_chooser_dialog_new (_("Choose a filename for the cdrom image"),
+					       NULL,
+					       GTK_FILE_CHOOSER_ACTION_SAVE,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       GTK_STOCK_OK, GTK_RESPONSE_OK,
+					       NULL);
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (chooser),
+					   "image.iso");
 
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
-			  "clicked",
-			  G_CALLBACK(file_sel_ok_cb),
-			  filesel);
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
-			  "clicked",
-			  G_CALLBACK(file_sel_cancel_cb),
-			  filesel);
-	g_signal_connect (G_OBJECT (filesel),
-			  "delete_event",
-			  G_CALLBACK(file_sel_delete_event_cb),
-			  filesel);
+	filename = NULL;
+	g_signal_connect (G_OBJECT (chooser),
+			  "response",
+			  G_CALLBACK(file_chooser_response_cb),
+			  &filename);
 
-	gtk_window_set_modal (GTK_WINDOW (filesel), TRUE);
+	gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
+	gtk_window_set_default_size (GTK_WINDOW (chooser), 600, 400);
 
-	gtk_widget_show (filesel);
+	gtk_widget_show (chooser);
 
 	/* Run in a loop untill we get the response from file selector */
 	gtk_main ();
 
-	data =  g_object_get_data (G_OBJECT (filesel), "file-name");
-
-	gtk_widget_destroy (filesel);
-
-	return (char *)data;
+	return filename;
 }
 
 static int
@@ -792,7 +774,7 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 
 	if (source_iso == NULL && !music_cd) {
 		if (filename != NULL) {
-			res = make_iso (filename, label, FALSE);
+			res = make_iso (filename, label, FALSE, TRUE);
 		} else {
 			res = RESULT_RETRY;
 		}
@@ -803,14 +785,14 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 			fd = mkstemp(filename);
 			close (fd);
 
-			res = make_iso (filename, label, FALSE);
+			res = make_iso (filename, label, FALSE, TRUE);
 			if (res == RESULT_RETRY) {
 				g_free (filename);
 				filename = g_build_filename (g_get_home_dir (), ".image.iso.XXXXXX", NULL);
 				fd = mkstemp(filename);
 				close (fd);
 
-				res = make_iso (filename, label, TRUE);
+				res = make_iso (filename, label, TRUE, TRUE);
 			}
 
 			if (!res) {
@@ -867,7 +849,7 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 			track->data.filename = g_strdup (filename);
 			tracks = g_list_prepend (tracks, track);
 		}
-		res = cd_recorder_write_tracks (cdrecorder, rec, tracks, speed, eject_cd, blank_cd, dummy);
+		res = cd_recorder_write_tracks (cdrecorder, rec, tracks, speed, eject_cd, blank_cd, dummy, FALSE);
 
 		if (source_iso == NULL) {
 			unlink (filename);
@@ -1007,11 +989,10 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 	GtkWidget *entry;
 	GtkWidget *reuse;
 	GtkWidget *music_check;
-	struct tm *tm;
 	int size_mb;
 	char *str;
 	char buf[129];
-	time_t now;
+	GDate *now;
 
 	label = glade_xml_get_widget (xml, "top_label");
 
@@ -1047,13 +1028,17 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 		gtk_entry_set_text (GTK_ENTRY (entry), disc_name);
 		gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
 	} else {
-		time (&now);
-		tm = localtime (&now);
+		now = g_date_new ();
+
+		g_date_set_time (now, time (NULL));
+
 		/*
 		translators: see strftime man page for meaning of %b, %d and %Y
 		the maximum length for this field is 32 characters
 		*/
-		strftime (buf, sizeof (buf), _("Personal Data, %b %d, %Y"), tm);	
+		g_date_strftime (buf, sizeof (buf), _("Personal Data, %b %d, %Y"), now);
+
+		g_date_free (now);
 		gtk_entry_set_text (GTK_ENTRY (entry), buf);
 	}
 
@@ -1161,7 +1146,7 @@ verify_iso (const char *filename, char **iso_label)
 	if (res != BUFFER_SIZE) {
 		return FALSE;
 	}
-	buf[BUFFER_SIZE+1] = '\0';
+	buf[BUFFER_SIZE] = '\0';
 	str = g_strdup (g_strstrip (buf));
 	if (!g_utf8_validate (str, -1, NULL)) {
 		/* Hmm, not UTF-8. Try the current locale. */
